#include <Wire.h>
#include <SHT1x.h>
#include <avr/sleep.h>
#include <avr/wdt.h>

#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#define dataPin  4 //D4
#define clockPin 3 //D3
#define SleepPin1 2 //D2

SHT1x sht1x(dataPin, clockPin);

  float temp_c; //температура град С
  float humidity; //влажность
int analogPin = 0;     // номер аналогового порта - делитель питания
int val = 0;           // переменная для хранения значения напряжения
byte i = 0; //счетчик для усредения
float voltage;
volatile boolean f_wdt=1;

//****************************************************************  
// перевод системы в режим сна
// система просыпается при переполнении watchdog
void system_sleep() {

  cbi(ADCSRA,ADEN);                    // выключим АЦП

  set_sleep_mode(SLEEP_MODE_PWR_DOWN); // здесь устанавливается режим сна
  sleep_enable();

  sleep_mode();                        // система засыпает

    sleep_disable();                     // система продолжает работу после переполнения watchdog
    sbi(ADCSRA,ADEN);                    // включаем АЦП

}

//****************************************************************
// 0=16ms, 1=32ms,2=64ms,3=128ms,4=250ms,5=500ms
// 6=1 sec,7=2 sec, 8=4 sec, 9= 8sec
void setup_watchdog(int ii) {

  byte bb;
  int ww;
  if (ii > 9 ) ii=9;
  bb=ii & 7;
  if (ii > 7) bb|= (1<<5);
  bb|= (1<<WDCE);
  ww=bb;
  MCUSR &= ~(1<<WDRF);
  // запуск таймера
  WDTCSR |= (1<<WDCE) | (1<<WDE);
  // установка периода срабатывания сторожевого таймера
  WDTCSR = bb;
  WDTCSR |= _BV(WDIE); //прерывание WDT - так в оригинале 
}
//****************************************************************  
// Обработка прерывания сторожевого таймера / выполняется при истечении установленного для него периода
ISR(WDT_vect) {
  f_wdt=1;  // установим глобальный флаг
}

void setup() 
{ 
  cbi( SMCR,SE );      // режим сна включён, тип - "power down"
  cbi( SMCR,SM0 );     // бит режима "power down"
  sbi( SMCR,SM1 );     // бит режима "power down"
  cbi( SMCR,SM2 );     // бит режима "power down"
 setup_watchdog(9); //8 c
  f_wdt=0;      // сбросим флаг
   pinMode(13, OUTPUT); //13 к светодиоду
   pinMode(SleepPin1, OUTPUT); //2 к sleep
analogReference(INTERNAL); //внутренний опорный источник 1,1 В 
Serial.begin(9600);
}//end setup

void loop() 
{
   if (f_wdt==1) 
  { f_wdt=0;      // сбросим флаг WDT
digitalWrite(13, HIGH); // включаем светодиод 
digitalWrite(SleepPin1, LOW); // выключаем sleep для Xbee
//digitalWrite(SleepPin2, LOW); // выключаем sleep для датчиков
delay(50);//ждем 
  temp_c = sht1x.readTemperatureC(); //температура
  humidity = sht1x.readHumidity(); //влажность
  Serial.print('O'); //0x4F
if (temp_c>=0) Serial.print("+");
if (temp_c<=0) Serial.print("-");
if (abs(temp_c)<10) Serial.print(" ");//выравниваем строку
  Serial.print(abs(temp_c), 1);
  Serial.print(' '); //0x20
  Serial.print(humidity, 0);
  Serial.print(' '); //0x20
  val = val+analogRead(analogPin);     // считываем значение напряжения питания
  i++; // увеличиваем счетчик на 1
  if (i>15) //каждый шестнадцатый раз 
  {
    val = val/i; //усредняем
    voltage = 0.00642*val; //калибровочный коэфф. при делителе 511/115 кОм (920 мВ)
    i = 0;
    val = 0;
  }
  Serial.println(voltage,2); // усредненная величина напряжения в вольтах
                           //первые 16 значений после запуска будут 0.00 
  delay(50);//ждем отправки последнего символа;                      
digitalWrite(13, LOW); // выключаем светодиод 
digitalWrite(SleepPin1, HIGH); // включаем sleep для Xbee 
  } //end if
system_sleep();

 } //end loop



